/*
FILE:     BigTableTest.C
*/
/*
VERSION:  8.050
*/
/*
DATE:     10/21/2013
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2013 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               RCSB PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/

/* This file can be used as a diagnostic tool for debugging the SSTable class,
   as well as for a tutorial on using the code.
   */


#include <string>
#include <vector>
#include <iostream>

#include "ISTable.h"


using std::string;
using std::vector;
using std::cout;
using std::endl;


#ifdef TWO
#define NUM_ROWS_IN_SINGLE_TABLE 50000
#define NUM_SINGLE_TABLES 4
#define NUM_COLS_IN_SINGLE_TABLE 100
#endif

#ifdef THREE
#define NUM_ROWS_IN_SINGLE_TABLE 10000
#define NUM_SINGLE_TABLES 20 
#define NUM_COLS_IN_SINGLE_TABLE 100
#endif

#ifdef FOUR
#define NUM_ROWS_IN_SINGLE_TABLE 1000
#define NUM_SINGLE_TABLES 200 
#define NUM_COLS_IN_SINGLE_TABLE 100
#endif

#ifndef FIVE
#define NUM_ROWS_IN_SINGLE_TABLE 100
#define NUM_SINGLE_TABLES 2000
#define NUM_COLS_IN_SINGLE_TABLE 100
#endif

ISTable* CreateTable(unsigned int numRows, unsigned int numCols);

int main(int argc, char** argv)
{

#ifndef VLAD_SMALL_TABLES

    // Create a big table 
    ISTable* bigTableP = CreateTable(NUM_SINGLE_TABLES *
      NUM_ROWS_IN_SINGLE_TABLE, NUM_COLS_IN_SINGLE_TABLE);

#else

    vector<ISTable*> tables;

    for (unsigned int tableI = 0; tableI < NUM_SINGLE_TABLES; ++tableI)
    {
        ISTable* isTableP = CreateTable(NUM_ROWS_IN_SINGLE_TABLE,
          NUM_COLS_IN_SINGLE_TABLE);
        tables.push_back(isTableP);
    }

#endif

    vector<string> searchCol;
    searchCol.push_back("10");
    searchCol.push_back("11");

    vector<string> targets;
#ifdef VLAD_ORIG_SEARCH
    targets.push_back("999");
    targets.push_back("999");
#else
    targets.push_back("180000");
    targets.push_back("180000");
#endif

#ifndef VLAD_SMALL_TABLES

    unsigned int bigIndex = bigTableP->FindFirst(targets, searchCol);
    if (bigIndex != bigTableP->GetNumRows())
    {
        cout << "BigTable: Found target at index " << bigIndex << endl;
    }
    else
    {
        cout << "BigTable: Did not find target" << endl;
    }

#else
    
    for (unsigned int tableI = 0; tableI < NUM_SINGLE_TABLES; ++tableI)
    {
        ISTable* isTableP = tables[tableI];

        unsigned int smallIndex = isTableP->FindFirst(targets, searchCol);
        if (smallIndex != isTableP->GetNumRows())
        {
            cout << "SmallTable: Table index " << tableI <<
              ", found target at index " << smallIndex << endl;
        }
        else
        {
            cout << "SmallTable: Table index " << tableI <<
              ", did not find target" << endl;
        }
    }

#endif

}


ISTable* CreateTable(unsigned int numRows, unsigned int numCols)
{

    ISTable* isTableP = new ISTable();

    for (unsigned int colI = 0; colI < numCols; ++colI)
    {
        isTableP->AddColumn(String::IntToString(colI));
    }
 
    for (unsigned int rowI = 0; rowI < numRows; ++rowI)
    {
        vector<string> newRow(numCols, String::IntToString(rowI));
        isTableP->AddRow(newRow);
    }

    cout << "CreateTable(): Created" << endl;

    return(isTableP);

}




#ifdef VLAD_OBSOLETE_DELETED

#include "SSTable.h"

using namespace std;

const int REDUNDANT_COL = 0;
const int SORTED_COL = 1;
const int NULL_COL = 2;
const int BACKWARDS_COL = 3;
const int SEMI_REDUNDANT_COL = 4;

const int REDUNDANCY_SIZE = 6;
const int COL_SIZE = 5;
const int ROW_SIZE = 100;

// prototypes
void PrintArray(vector<int> *);
void generateRedundantColumn(vector<string> &, int);
void generateSortedColumn(vector<string> &, int);
void generateNullColumn(vector<string> &, int);
void generateBackwardsColumn(vector<string> &, int);
void generateSemiRedundantColumn(vector<string> &, int);
void generateSSTable(SSTable *, int, int);
int Assert_All(SSTable *);
int CompareTables(SSTable & a, SSTable & b);
int CompareStringArrays(vector<string> * a,
                        vector<string> * b);

void main(int argc, char** argv) {
  SSTable *ss, *ss2, *ss3;
  int c = 0, r = 0, j;

  if (argc > 1) {
    c = atoi(argv[1]);
    if (argc > 2) {
      r = atoi(argv[2]);
    } else {
      r = ROW_SIZE;
    }
  } else {
    c = COL_SIZE;
    r = ROW_SIZE;
  } 

  if (c < 5) c = 5;
  if (r <= REDUNDANCY_SIZE) r = REDUNDANCY_SIZE + 1;
  string cs("RANDOM"); // a scrap string
  vector<string> aCol; // a scrap column
  vector<int> * list1 = NULL, *list2=NULL, *list3=NULL, *list4=NULL;
  int errCode, len;

  ss = new SSTable(); // default constructor
  ss2 = new SSTable();
  ss3 = new SSTable("TestTable3");  // other constructor
  ss->Rename("TestTable1");
  ss->Rename("TestTable2");

  // generate initial tables, all columns searchable
  generateSSTable(ss, c, r/2);
  cout << "Generated Table of " << ss->GetNumColumns() << " columns and "<< ss->GetNumRows() << " rows" << endl;
  generateSSTable(ss2, c, r);
  cout << "Generated Table of " << ss2->GetNumColumns() << " columns and " << ss2->GetNumRows() << " rows" << endl;
  generateSSTable(ss3, c, 2*r);
  cout << "Generated Table of " << ss3->GetNumColumns() << " columns and " << ss3->GetNumRows() << " rows" << endl;
  generateSemiRedundantColumn(aCol, r);
  cs.clear();
  cs += ((r-1) / REDUNDANCY_SIZE);

  list1 = ss->SearchColumn(cs, SEMI_REDUNDANT_COL, errCode);
  if (errCode < NO_TABLE_ERROR)
    cerr << SSTable::GetErrorMessage(errCode) << endl;
  if (!list1) len = 0;
  else len = list1->size();
  if (len != 0)
    cerr << "Table one shouldn't have any " << cs << endl;

  list2 = ss2->SearchColumn(cs, SEMI_REDUNDANT_COL, errCode);
  if (errCode < NO_TABLE_ERROR)
    cerr << SSTable::GetErrorMessage(errCode) << endl;
  if (!list2) len = 0;
  else len = list2->size();

  if (len != (r % REDUNDANCY_SIZE)) {
    if ((len != REDUNDANCY_SIZE) && ((r % REDUNDANCY_SIZE) == 0))
      cerr << "Table 2 has wrong number of " << cs << endl;
  }

  list3 = ss3->SearchColumn(cs, SEMI_REDUNDANT_COL, errCode);
  if (errCode < NO_TABLE_ERROR)
    cerr << SSTable::GetErrorMessage(errCode) << endl;
  if (!list3) len = 0;
  else len = list3->size();

  if (len != REDUNDANCY_SIZE)
    cerr << "Table 3 has wrong number of " << cs << endl;

  cout << "Passed SearchColumn Test" << endl;
  
  if (list1) delete list1; list1 = NULL;
  if (list2) delete list2; list2 = NULL;
  if (list3) delete list3; list3 = NULL;

  //  This is an example of searching for a range of values
  int precis = ((int) (log10(r))) + 1;
  // third column gets >= ceil(log(r)) decimal places 
  ss->SetPrecision(precis, 3);
  ss2->SetPrecision(precis, 3);
  ss3->SetPrecision(precis, 3);

  ss->SetFlags(SSTable::DT_DOUBLE, 3); // set column 3 to datatype double
  ss2->SetFlags(SSTable::DT_DOUBLE, 3);
  ss3->SetFlags(SSTable::DT_DOUBLE, 3);

  double x = (double) r;
  double x2 = 0.0000;
  char buff[80], buff2[80];
  sprintf(buff, "%g", x);
  sprintf(buff2, "%g", x2);
  string buffString(buff);
  string buff2String(buff2);

  // Let's find all rows with values 0 <= value < r

  list1 = ss->ConstrainColumnLessThan(buffString, 3, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  list2 = ss->ConstrainColumnGreaterThanEqual(buff2String, 3, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  list3 = SSTable::SetIntersect(list1, list2);
  if (!list3) cerr << "No Matches?" << endl;
  if (list3->size() != r/2) cerr << "Failed Constrain Test, part 1" << endl;
  else cout << "Passed Constrain Test, part 1" << endl;

  if (list1) delete list1; list1 = NULL;
  if (list2) delete list2; list2 = NULL;
  if (list3) delete list3; list3 = NULL;

  list1 = ss2->ConstrainColumnLessThan(buffString, 3, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  list2 = ss2->ConstrainColumnGreaterThanEqual(buff2String, 3, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  list3 = SSTable::SetIntersect(list1, list2);
  if (!list3) cerr << "No Matches?" << endl;
  if (list3->size() != r) cerr << "Failed Constrain Test, part 2" << endl;
  else cout << "Passed Constrain Test, part 2" << endl;

  if (list1) delete list1; list1 = NULL;
  if (list2) delete list2; list2 = NULL;
  if (list3) delete list3; list3 = NULL;
  list1 = ss3->ConstrainColumnLessThan(buffString, 3, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  list2 = ss3->ConstrainColumnGreaterThanEqual(buff2String, 3, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  list3 = SSTable::SetIntersect(list1, list2);
  if (!list3) cerr << "No Matches?" << endl;
  if (list3->size() != r) cerr << "Failed Constrain Test, part 3" << endl;
  else cout << "Passed Constrain Test, part 3" << endl;

  if (list1) delete list1; list1 = NULL;
  if (list2) delete list2; list2 = NULL;
  if (list3) delete list3; list3 = NULL;

  if ((Assert_All(ss) < 0) || (Assert_All(ss2) < 0) || (Assert_All(ss3) < 0))
    cout << "Failed Constrain Test" << endl;
  else
    cout << "Passed Constrain Test" << endl;

  if (list1) delete list1; list1 = NULL;
  if (list2) delete list2; list2 = NULL;
  if (list3) delete list3; list3 = NULL;

  // The next test shows how to search multiple columns

  string empty("");
  string target;
  target += (10000 + r/2);
  int cIndex;
  cIndex = ss->GetColumnIndex("NC2");
  if (cIndex < 0) cerr << "ERROR: problem in getting column index on column NC2";
  ss->SetFlags(SSTable::DT_INTEGER, 1);
  list1 = ss->ConstrainColumnLessThan(target, 1, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  list2 = ss->ConstrainColumnGreaterThanEqual(buff2String, 1, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  list3 = SSTable::SetIntersect(list1, list2);
  if (!list3) cerr << "No Matches?" << endl;
  if (list1) delete list1; list1 = NULL; // reset lists
  if (list2) delete list2; list2 = NULL;
  list1 = ss->ConstrainColumnGreaterThanEqual(empty, cIndex, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  // Now list1 and list3 have results from two different columns, now join them
  list2 = SSTable::SetIntersect(list1, list3);
  list4 = SSTable::SetUnion(list1, list3);
  if ((!list3) || (!list4) || !(list2))
    cerr << "Failed 2nd Constrain Test" << endl;
  else if (list3->size() != r/2) cerr << "Failed 2nd Constrain Test" << endl;
  else if (list4->size() != r/2) cerr << "Failed 2nd Constrain Test" << endl;
  else if (list2->size() != r/2) cerr << "Failed 2nd Constrain Test" << endl;
  else cout << "Passed 2nd Constrain Test, part 1" << endl;

  if (list1) delete list1; list1 = NULL;
  if (list2) delete list2; list2 = NULL;
  if (list3) delete list3; list3 = NULL;
  if (list4) delete list4; list4 = NULL;

  ss2->SetFlags(SSTable::DT_INTEGER, 1);
  list1 = ss2->ConstrainColumnLessThan(target, 1, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  list2 = ss2->ConstrainColumnGreaterThanEqual(buff2String, 1, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  list3 = SSTable::SetIntersect(list1, list2);
  if (!list3) cerr << "No Matches?" << endl;
  if (list1) delete list1; list1 = NULL; // reset lists
  if (list2) delete list2; list2 = NULL;
  list1 = ss2->ConstrainColumnGreaterThanEqual(empty, cIndex, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  // Now list1 and list3 have results from two different columns, now join them
  list2 = SSTable::SetIntersect(list1, list3);
  list4 = SSTable::SetUnion(list1, list3);
  if ((!list3) || (!list4)) cerr << "Failed 2nd Constrain Test" << endl;
  else if (list3->size() != r/2) cerr << "Failed 2nd Constrain Test" << endl;
  else if (list4->size() != r) cerr << "Failed 2nd Constrain Test" << endl;
  else if (list2->size() != r/2) cerr << "Failed 2nd Constrain Test" << endl;
  else cout << "Passed 2nd Constrain Test, part 2" << endl;

  if (list1) delete list1; list1 = NULL;
  if (list2) delete list2; list2 = NULL;
  if (list3) delete list3; list3 = NULL;
  if (list4) delete list4; list4 = NULL;

  ss3->SetFlags(SSTable::DT_INTEGER, 1);
  list1 = ss3->ConstrainColumnLessThan(target, 1, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  list2 = ss3->ConstrainColumnGreaterThanEqual(buff2String, 1, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  list3 = SSTable::SetIntersect(list1, list2);
  if (!list3) cerr << "No Matches?" << endl;
  if (list1) delete list1; list1 = NULL; // reset lists
  if (list2) delete list2; list2 = NULL;
  list1 = ss3->ConstrainColumnGreaterThanEqual(empty, cIndex, errCode);
  if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
  // Now list1 and list3 have results from two different columns, now join them
  list2 = SSTable::SetIntersect(list1, list3);
  list4 = SSTable::SetUnion(list1, list3);
  if ((!list3) || (!list4)) cerr << "Failed 2nd Constrain Test" << endl;
  else if (list3->size() != r/2) cerr << "Failed 2nd Constrain Test" << endl;
  else if (list4->size() != ss3->GetNumRows())
    cerr << "Failed 2nd Constrain Test" << endl;
  else if (list2->size() != r/2) cerr << "Failed 2nd Constrain Test" << endl;
  else cout << "Passed 2nd Constrain Test, part 3" << endl;

  if (list1) delete list1; list1 = NULL;
  if (list2) delete list2; list2 = NULL;
  if (list3) delete list3; list3 = NULL;
  if (list4) delete list4; list4 = NULL;

  // Persistence Tests ****************************************************
  FileNavigator * f = new FileNavigator;
  f->OpenFile("test.tdb", WRITE_MODE);
  f->ReadFileHeader(); // always must be done
  Word list_loc = 0, place = 0;
  errCode = f->WriteWord(0, list_loc); // we'll update this later
  if (errCode < 0) f->PrintError(errCode);
  int table_locs[10];

  // write 10 tables
  for (j = 0; j < 10; j++) {
    switch (j%3) {
    case 0: table_locs[j] = ss->WriteObject(f); break;
    case 1: table_locs[j] = ss2->WriteObject(f); break;
    case 2: table_locs[j] = ss3->WriteObject(f); break;
    default: cerr << "Failed persistence test" << endl;
    }
  }

  // now that the locations are in table_locs, write it out and update
  // list_loc.  the table names can also be written had they been kept track
  // of.  Normally, one might want to construct a name list and a parallel
  // table list.  list_loc would point to the first of these 2 lists, and the
  // second such list would be list_loc + 1
  errCode = f->WriteWords((Word *) table_locs, 10, place);
  if (errCode < 0) f->PrintError(errCode);
  errCode = f->UpdateWord(place, list_loc, list_loc);
  if (list_loc != 0) cerr << "Update failed in FileNavigator" << endl;
  if (errCode < 0) f->PrintError(errCode);
  f->CloseFile();
  cout << "Passed Persistence Test, part 1" << endl;

  if (f) delete f;
  f = new FileNavigator;
  f->OpenFile("test.tdb", READ_MODE);
  f->ReadFileHeader();
  SSTable * t = NULL;
  Word where;
  where = f->GetWord(0, errCode); // the location of the list
  if (errCode < 0) f->PrintError(errCode);
  uWord numT = 0;
  int * p = (int *) f->GetWords(where, numT, errCode);
  if (errCode < 0) f->PrintError(errCode);
  if (numT > 0) 
    t = new SSTable[numT];

  // Get tables in reverse order
  for (j = (int) (numT - 1); j >= 0; j--) {
    errCode = t[j].GetObject(p[j], f);
    if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode) << endl;

    t[j].SetFlags(SSTable::DT_INTEGER, 1);
    list1 = t[j].ConstrainColumnLessThan(target, 1, errCode);
    if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
    list2 = t[j].ConstrainColumnGreaterThanEqual(buff2String, 1, errCode);
    if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
    list3 = SSTable::SetIntersect(list1, list2);
    if (!list3) cerr << "No Matches?" << endl;
    if (list1) delete list1; list1 = NULL; // reset lists
    if (list2) delete list2; list2 = NULL;
    list1 = t[j].ConstrainColumnGreaterThanEqual(empty, cIndex, errCode);
    if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
    // Now list1 and list3 have results from two different columns,
    // now join them
    list2 = SSTable::SetIntersect(list1, list3);
    list4 = SSTable::SetUnion(list1, list3);

    if ((!list3) || (!list4))
      cerr << "Failed 3rd Constrain Test, object " << j << endl;
    else if (list3->size() != r/2)
      cerr << "Failed 3rd Constrain Test, object " << j << endl;
    else if (list4->size() != t[j].GetNumRows())
      cerr << "Failed 3rd Constrain Test, object " << j << endl;
    else if (list2->size() != r/2)
      cerr << "Failed 3rd Constrain Test, object " << j << endl;
    else cout << "Passed 3rd Constrain Test, object " << j << endl;

    if (list1) delete list1; list1 = NULL;
    if (list2) delete list2; list2 = NULL;
    if (list3) delete list3; list3 = NULL;
    if (list4) delete list4; list4 = NULL;

    list1 = t[j].ConstrainColumnLessThan(buffString, 3, errCode);
    if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
    list2 = t[j].ConstrainColumnGreaterThanEqual(buff2String, 3, errCode);
    if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode);
    list3 = SSTable::SetIntersect(list1, list2);
    if (!list3) cerr << "No Matches?" << endl;

    switch(j % 3) {
    case 0:
      errCode = CompareTables(*ss, t[j]);
      if (errCode < 0)
        cerr << "Failed: Tables non-equivalent to parents" << endl;
      else cout << "Passed Table Equivalency Test" << endl;

      if (list3->size() != r/2)
        cerr << "Failed Secondary Constrain Test, object " << j << endl;
      else cout << "Passed Secondary Constrain Test, object " << j << endl;

      break;

    case 1: 
      errCode = CompareTables(*ss2, t[j]);
      if (errCode < 0)
        cerr << "Failed: Tables non-equivalent to parents" << endl;
      else cout << "Passed Table Equivalency Test" << endl;

      if (list3->size() != r)
        cerr << "Failed Secondary Constrain Test, object " << j << endl;
      else cout << "Passed Secondary Constrain Test, object " << j << endl;

      break;

    case 2:
      errCode = CompareTables(*ss3, t[j]);
      if (errCode < 0)
        cerr << "Failed: Tables non-equivalent to parents" << endl;
      else cout << "Passed Table Equivalency Test" << endl;

      if (list3->size() != r)
        cerr << "Failed Secondary Constrain Test, object " << j << endl;
      else cout << "Passed Secondary Constrain Test, object " << j << endl;

      break;

    default: cerr << "Error in Getting Tables" << endl;
    }
    if (errCode < 0) cerr << SSTable::GetErrorMessage(errCode) << endl;

    if (list1) delete list1; list1 = NULL;
    if (list2) delete list2; list2 = NULL;
    if (list3) delete list3; list3 = NULL;
    errCode = Assert_All(&t[j]);
    if (errCode < 0) cerr << "Failed Assertion, object " << j << endl;

  }

} // MAIN


void PrintArray(vector<int> * target) {
  if (target) {
    int len = target->size();
    for (int i = 0; i < len - 1; i++) {
      cout << (*target)[i] << ", ";
    }
    if (len > 0)
      cout << (*target)[len - 1];
  }
  cout << endl;
}

void generateRedundantColumn(vector<string> & col, int num) {
   int i;
   string * cs;

   for (i = 0; i < num; i++) {
     cs = new string("1000");
     col.push_back(*cs);
	 if (cs) delete cs;
   }
}   

void generateSortedColumn(vector<string> & col, int num) {
  int i;
  string * cs;

  for (i = 0; i < num; i++) {
    cs = new string("");
    (*cs) += (i + 10000);
    col.push_back(*cs);
	 if (cs) delete cs;
  }

}

void generateNullColumn(vector<string> & col, int num) {
   int i;
   string cs;

   col.clear();

   cs = "";
   for (i = 0; i < num; i++) {
     col.push_back(cs);
   }

}   

void generateBackwardsColumn(vector<string> & col, int num) {
   int i;
   string cs;

   col.clear();

   for (i = num - 1; i >= 0; i--) {
     cs += i;
     cs += ".";
     cs += abs(10 - i);
     col.push_back(cs);
     cs.clear();
   }

}   

void generateSemiRedundantColumn(vector<string> & col, int num) {
   int i, j, d, m;
   string cs;

   col.clear();

   d = num / REDUNDANCY_SIZE;
   m = num % REDUNDANCY_SIZE;
   for (i = 0; i < d; i++) {
     for (j = 0; j < REDUNDANCY_SIZE; j++) {
       cs += i;
       col.push_back(cs);
       cs.clear();
     }
   }
   for (j = 0; j < m; j++) {
     cs += i;
     col.push_back(cs);
     cs.clear();
   }

}   

void generateSSTable(SSTable * ss, int numCols, int numRows) {
  vector<string> * col;

  string * cs;
  int i, col_type;

  cs = NULL;

  if (!ss) {
    cout << "Error" << endl;
    return(1);
  }

  for (i = 0; i < numCols; i++) {
    col_type = i % 5;
    switch (col_type) {
      case REDUNDANT_COL:
        col = new vector<string>;

       	generateRedundantColumn(*col, numRows);
//       	if (cs) cs->clear();
	cs = new string("RC");
	*cs += i;
        ss->InsertColumn(*col, *cs, i, 1);
		  if (col) delete col;
       	if (cs) delete cs;
        break;

      case SORTED_COL:
        col = new vector<string>;
	generateSortedColumn(*col, numRows);
//       	if (cs) cs->clear();
	cs = new string("SC");
	*cs += i;
        ss->InsertColumn(*col, *cs, i, 1);
		  if (col) delete col;
       	if (cs) delete cs;
        break;

      case NULL_COL:
        col = new vector<string>;
	generateNullColumn(*col, numRows);
//       	if (cs) cs->clear();
	cs = new string("NC");
	*cs += i;
        ss->InsertColumn(*col, *cs, i, 1);
		  if (col) delete col;
       	if (cs) delete cs;
        break;

      case BACKWARDS_COL:
        col = new vector<string>;
	generateBackwardsColumn(*col, numRows);
//       	if (cs) cs->clear();
	cs = new string("BC");
	*cs += i;
        ss->InsertColumn(*col, *cs, i, 1);
		  if (col) delete col;
       	if (cs) delete cs;
        break;


      case SEMI_REDUNDANT_COL:
        col = new vector<string>;
	generateSemiRedundantColumn(*col, numRows);
//       	if (cs) cs->clear();
	cs = new string("SRC");
	*cs += i;
        ss->InsertColumn(*col, *cs, i, 1);
		  if (col) delete col;
       	if (cs) delete cs;
        break;

      default: cout << "Error";
	       return(0);
    }
  }

}

int Assert_All(SSTable * s) {
  int err;
  if (!s) {
    cerr << "Assertion error: No table" << endl;
    return -1;
  }
  for (int i = s->GetNumColumns() - 1; i >= 0; i--) {
    err = s->Assert(i);
    if (err < 0) {
      cerr << "ASSERT_ALL WARNING ON COLUMN " << i << ": " << SSTable::GetErrorMessage(err) << endl;
      return -1;
    }
  }
  return NO_TABLE_ERROR;
}

int CompareTables(SSTable & a, SSTable & b) {

  vector<string> * a1 = NULL, * b1 = NULL;
  int x1, x2, y1, y2;
  x1 = a.GetNumRows();
  x2 = b.GetNumRows();
  if (x1 != x2) {
    cerr << "Tables non-equivalent" << endl;
    return -1;
  }

  y1 = a.GetNumColumns();
  y2 = b.GetNumColumns();

  if (y1 != y2) {
    cerr << "Tables non-equivalent" << endl;
    return -2;
  }

  a1 = a.GetColumnNames();
  b1 = b.GetColumnNames();

  if (CompareStringArrays(a1, b1)) {
    cerr << "Tables non-equivalent" << endl;
    return -3;
  }
  if (a1) delete a1; a1 = NULL;
  if (b1) delete b1; b1 = NULL;

  int i;
  for (i = 0; i < x1; i++) {
    a1 = a.GetRow(i);
    b1 = b.GetRow(i);
    if (CompareStringArrays(a1, b1)) {
     cerr << "Tables non-equivalent" << endl;
     return (-4 - i);
    }
    if (a1) delete a1; a1 = NULL;
    if (b1) delete b1; b1 = NULL;
  }

  for (i = 0; i < y1; i++) {
    a1 = a.GetColumn(i);
    b1 = b.GetColumn(i);
    if (CompareStringArrays(a1, b1)) {
     cerr << "Tables non-equivalent" << endl;
     return (-4 - x1 - i);
    }
    if (a1) delete a1; a1 = NULL;
    if (b1) delete b1; b1 = NULL;
  }
  return 1;
}

int CompareStringArrays(vector<string> * a,
                        vector<string> * b) {
  if ((!a) && (!b)) return 0;
  if (!a) return -1;
  if (!b) return -2;
  int alen = a->size();
  int blen = b->size();
  if (alen != blen) return -3;
  for (int i = 0; i < alen; i++) {
    if ((*a)[i] != (*b)[i])
      return (-4 - i);
  }

  return 0;
}

#endif
